# Mentorenwahl: A fullstack application for assigning mentors to students based on their whishes.
# Copyright (C) 2022  Dominic Grimm
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

require "jwt"

module Backend
  module Api
    # Authorization and authentication utilities
    module Auth
      extend self

      # Bearer token header
      BEARER = "Bearer "

      # Creates raw JWT token
      #
      # WARNING: Always use a wrapper for this method
      private def create_jwt(data, expiration : Int) : String
        JWT.encode({"data" => data.to_h, "exp" => expiration}, Backend.config.api.jwt_secret, JWT::Algorithm::HS256)
      end

      # Decodes JWT token
      def decode_jwt(jwt : String) : JSON::Any
        JWT.decode(jwt, Backend.config.api.jwt_secret, JWT::Algorithm::HS256)[0]
      end

      # :ditto:
      def decode_jwt?(jwt : String) : JSON::Any?
        decode_jwt(jwt)
      rescue
        nil
      end

      # Creates JWT token for user
      def create_user_jwt(user_id : Int, expiration : Int) : String
        create_jwt({user_id: user_id}, expiration)
      end
    end
  end
end
