# Mentorenwahl: A fullstack application for assigning mentors to students based on their whishes.
# Copyright (C) 2022  Dominic Grimm
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

module Backend
  module Db
    # Teacher vote model
    class TeacherVote < Granite::Base
      table teacher_votes

      belongs_to :vote
      belongs_to :teacher

      # Teacher votes's ID
      column id : Int64, primary: true

      # Teacher vote's priority
      column priority : Int32

      validate :teacher, "must be vote unique" do |teacher_vote|
        self.where(vote_id: teacher_vote.vote.id, teacher_id: teacher_vote.teacher.not_nil!.id).count.run.as(Int64).zero?
      end

      validate :priority, "must be positive" do |teacher_vote|
        teacher_vote.priority >= 0
      end

      validate :priority, "must be less than the number of teachers" do |teacher_vote|
        teacher_vote.priority < Teacher.count
      end

      validate :priority, "must be vote unique" do |teacher_vote|
        self.where(vote_id: teacher_vote.vote.id, priority: teacher_vote.priority).count.run.as(Int64).zero?
      end
    end
  end
end
