# Mentorenwahl: A fullstack application for assigning mentors to students based on their whishes.
# Copyright (C) 2022  Dominic Grimm
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

module Backend
  module Worker
    module Jobs
      # Peridically caches user data in redis cache
      class CacheLdapUsersJob < Mosquito::PeriodicJob
        run_every Backend.config.ldap.cache_refresh_interval.minutes

        # :ditto:
        def perform : Nil
          Redis::CLIENT.keys("ldap:user:*")
            .map(&.as(String).split(":")[2].to_i)
            .concat(Db::User.all.map(&.id.not_nil!.to_i))
            .uniq!
            .each do |id|
              spawn do
                CacheLdapUserJob.new(id).enqueue
              end
            end

          Fiber.yield
        end
      end
    end
  end
end
